@@ -11,12 +11,14 @@ import operator
 from lightwood.mixers.helpers.default_net import DefaultNet
 from lightwood.mixers.helpers.transformer import Transformer
 from lightwood.mixers.helpers.ranger import Ranger
+from lightwood.mixers.helpers.transform_corss_entropy_loss import TransformCrossEntropyLoss
 from lightwood.config.config import CONFIG
+from lightwood.constants.lightwood import COLUMN_DATA_TYPES
 
 
 class NnMixer:
-    def __init__(self, dynamic_parameters, is_categorical_output=False):
-        self.is_categorical_output = is_categorical_output
+    def __init__(self, dynamic_parameters):
+        self.output_types = None
         self.net = None
         self.optimizer = None
         self.input_column_names = None
@@ -25,7 +27,8 @@ class NnMixer:
         self.encoders = None
         self.optimizer_class = None
         self.optimizer_args = None
-        self.criterion = None
+        self.criterion_arr = None
+        self.unreduced_criterion_arr = None
 
         self.batch_size = 200
         self.epochs = 120000
@@ -39,6 +42,8 @@ class NnMixer:
         self.is_selfaware = False
         self.last_unaware_net = False
 
+        self.max_confidence_per_output = []
+
         self.monitor = None
         for k in CONFIG.MONITORING:
             if CONFIG.MONITORING[k]:
@@ -52,6 +57,52 @@ class NnMixer:
             'sampler': None
         }
 
+    def build_confidence_normalization_data(self, ds, subset_id=None):
+        """
+        :param ds:
+        :return:
+        """
+        self.net = self.net.eval()
+
+        ds.encoders = self.encoders
+        ds.transformer = self.transformer
+
+        data_loader = DataLoader(ds, batch_size=self.batch_size, shuffle=True, num_workers=0)
+
+        loss_confidence_arr = []
+
+        for i, data in enumerate(data_loader, 0):
+            inputs, labels = data
+            inputs = inputs.to(self.net.device)
+            labels = labels.to(self.net.device)
+
+            with torch.no_grad():
+                if self.is_selfaware:
+                    outputs, awareness = self.net(inputs)
+                else:
+                    outputs = self.net(inputs)
+
+            loss = None
+            for k, criterion in enumerate(self.criterion_arr):
+                if len(loss_confidence_arr) <= k:
+                    loss_confidence_arr.append([])
+                    self.max_confidence_per_output.append(None)
+
+                try:
+                    confidences = criterion.estimate_confidence(outputs[:,ds.out_indexes[k][0]:ds.out_indexes[k][1]])
+                    loss_confidence_arr[k].extend(confidences)
+                except:
+                    pass
+
+        for k, _ in enumerate(self.criterion_arr):
+            if len(loss_confidence_arr[k]) > 0:
+                loss_confidence_arr[k] = np.array(loss_confidence_arr[k])
+                nf_pct = np.percentile(loss_confidence_arr[k], 95)
+                self.max_confidence_per_output[k] = max(loss_confidence_arr[k][loss_confidence_arr[k] < nf_pct])
+
+        return True
+
+
     def fit(self, ds=None, callback=None):
 
         ret = 0
@@ -67,6 +118,8 @@ class NnMixer:
         """
         when_data_source.transformer = self.transformer
         when_data_source.encoders = self.encoders
+        _, _ = when_data_source[0]
+
         data_loader = DataLoader(when_data_source, batch_size=self.batch_size, shuffle=False, num_workers=0)
 
         # set model into evaluation mode in order to skip things such as Dropout
@@ -74,6 +127,8 @@ class NnMixer:
 
         outputs = []
         awareness_arr = []
+        loss_confidence_arr = [[]] * len(when_data_source.out_indexes)
+
         for i, data in enumerate(data_loader, 0):
             inputs, _ = data
             inputs = inputs.to(self.net.device)
@@ -82,11 +137,22 @@ class NnMixer:
                 if self.is_selfaware:
                     output, awareness = self.net(inputs)
                     awareness = awareness.to('cpu')
-                    awareness_arr.extend(awareness)
+                    awareness_arr.extend(awareness.tolist())
                 else:
                     output = self.net(inputs)
                     awareness_arr = None
 
+                for k, criterion in enumerate(self.criterion_arr):
+                    try:
+                        max_conf = 1
+                        if len(self.max_confidence_per_output) >= (k - 1) and self.max_confidence_per_output[k] is not None:
+                            max_conf = self.max_confidence_per_output[k]
+
+                        confidences = criterion.estimate_confidence(output[:,when_data_source.out_indexes[k][0]:when_data_source.out_indexes[k][1]], max_conf)
+                        loss_confidence_arr[k].extend(confidences)
+                    except Exception as e:
+                        loss_confidence_arr[k] = None
+
                 output = output.to('cpu')
 
             outputs.extend(output)
@@ -95,20 +161,6 @@ class NnMixer:
         confidence_trasnformed_vectors = {}
 
         for i in range(len(outputs)):
-            if awareness_arr is not None:
-                confidence_vector = awareness_arr[i]
-                transformed_confidence_vectors = when_data_source.transformer.revert(
-                    confidence_vector, feature_set='output_features')
-                for feature in transformed_confidence_vectors:
-                    if feature not in confidence_trasnformed_vectors:
-                        confidence_trasnformed_vectors[feature] = []
-                    # @TODO: Very simple algorithm to get a confidence from the awareness,
-                    # not necessarily what we want for the final version
-                    confidence_trasnformed_vectors[feature] += [
-                        1 - sum(np.abs(
-                            transformed_confidence_vectors[feature])) / len(transformed_confidence_vectors[feature])
-                    ]
-
             output_vector = outputs[i]
             transformed_output_vectors = when_data_source.transformer.revert(
                 output_vector, feature_set='output_features')
@@ -118,14 +170,17 @@ class NnMixer:
                 output_trasnformed_vectors[feature] += [transformed_output_vectors[feature]]
 
         predictions = {}
-        for output_column in output_trasnformed_vectors:
+        for k, output_column in enumerate(list(output_trasnformed_vectors.keys())):
             decoded_predictions = when_data_source.get_decoded_column_data(
                 output_column,
                 when_data_source.encoders[output_column]._pytorch_wrapper(output_trasnformed_vectors[output_column])
             )
             predictions[output_column] = {'predictions': decoded_predictions}
             if awareness_arr is not None:
-                predictions[output_column]['confidences'] = confidence_trasnformed_vectors[output_column]
+                predictions[output_column]['selfaware_confidences'] = [1/x[k] for x in awareness_arr]
+
+            if loss_confidence_arr[k] is not None:
+                predictions[output_column]['loss_confidences'] = loss_confidence_arr[k]
 
             if include_encoded_predictions:
                 predictions[output_column]['encoded_predictions'] = output_trasnformed_vectors[output_column]
@@ -170,19 +225,21 @@ class NnMixer:
             inputs = inputs.to(self.net.device)
             labels = labels.to(self.net.device)
 
-            if self.is_categorical_output:
-                target = labels.cpu().numpy()
-                target_indexes = np.where(target > 0)[1]
-                targets_c = torch.LongTensor(target_indexes)
-                labels = targets_c.to(self.net.device)
-
             with torch.no_grad():
                 if self.is_selfaware:
                     outputs, awareness = self.net(inputs)
                 else:
                     outputs = self.net(inputs)
 
-            loss = self.criterion(outputs, labels)
+            loss = None
+            for k, criterion in enumerate(self.criterion_arr):
+                target_loss = criterion(outputs[:,ds.out_indexes[k][0]:ds.out_indexes[k][1]], labels[:,ds.out_indexes[k][0]:ds.out_indexes[k][1]])
+
+                if loss is None:
+                    loss = target_loss
+                else:
+                    loss += target_loss
+
             running_loss += loss.item()
             error = running_loss / (i + 1)
 
@@ -236,16 +293,6 @@ class NnMixer:
         """
         if initialize:
             self.fit_data_source(ds)
-            if self.is_categorical_output:
-                # The WeightedRandomSampler samples "randomly" but can assign higher weight to certain rows, we assign each rows it's weight based on the target variable value in that row and it's associated weight in the output_weights map (otherwise used to bias the loss function)
-                if ds.output_weights is not None and ds.output_weights is not False and CONFIG.OVERSAMPLE:
-                    weights = []
-                    for row in ds:
-                        _, out = row
-                        # @Note: This assumes one-hot encoding for the encoded_value
-                        weights.append(ds.output_weights[torch.argmax(out).item()])
-
-                    self._nonpersistent['sampler'] = torch.utils.data.WeightedRandomSampler(weights=weights,num_samples=len(weights),replacement=True)
 
             self.net = self.nn_class(ds, self.dynamic_parameters, selfaware=False)
             self.net = self.net.train()
@@ -255,15 +302,23 @@ class NnMixer:
 
             self.awareness_criterion = torch.nn.MSELoss()
 
-            if self.criterion is None:
-                if self.is_categorical_output:
-                    if ds.output_weights is not None and ds.output_weights is not False and not CONFIG.OVERSAMPLE:
-                        output_weights = torch.Tensor(ds.output_weights).to(self.net.device)
-                    else:
-                        output_weights = None
-                    self.criterion = torch.nn.CrossEntropyLoss(weight=output_weights)
+            if self.criterion_arr is None:
+                self.criterion_arr = []
+                self.unreduced_criterion_arr = []
+                if ds.output_weights is not None and ds.output_weights is not False:
+                    output_weights = torch.Tensor(ds.output_weights).to(self.net.device)
                 else:
-                    self.criterion = torch.nn.MSELoss()
+                    output_weights = None
+                for output_type in ds.out_types:
+                    if output_type in (COLUMN_DATA_TYPES.CATEGORICAL):
+                        self.criterion_arr.append(TransformCrossEntropyLoss(weight=output_weights))
+                        self.unreduced_criterion_arr.append(TransformCrossEntropyLoss(weight=output_weights,reduce=False))
+                    elif output_type in (COLUMN_DATA_TYPES.NUMERIC):
+                        self.criterion_arr.append(torch.nn.MSELoss())
+                        self.unreduced_criterion_arr.append(torch.nn.MSELoss(reduce=False))
+                    else:
+                        self.criterion_arr.append(torch.nn.MSELoss())
+                        self.unreduced_criterion_arr.append(torch.nn.MSELoss(reduce=False))
 
             self.optimizer_class = Ranger
             if self.optimizer_args is None:
@@ -333,34 +388,46 @@ class NnMixer:
                 else:
                     outputs = self.net(inputs)
 
-                if self.is_categorical_output:
-                    target = labels.cpu().numpy()
-                    target_indexes = np.where(target > 0)[1]
-                    targets_c = torch.LongTensor(target_indexes)
-                    cat_labels = targets_c.to(self.net.device)
-                    loss = self.criterion(outputs, cat_labels)
-                else:
-                    loss = self.criterion(outputs, labels)
+                loss = None
+                for k, criterion in enumerate(self.criterion_arr):
+                    target_loss = criterion(outputs[:,ds.out_indexes[k][0]:ds.out_indexes[k][1]], labels[:,ds.out_indexes[k][0]:ds.out_indexes[k][1]])
+                    if loss is None:
+                        loss = target_loss
+                    else:
+                        loss += target_loss
 
+                awareness_loss = None
                 if self.is_selfaware:
-                    real_loss = torch.abs(labels - outputs) # error precentual to the target
-                    real_loss = torch.Tensor(real_loss.tolist()) # disconnect from the graph (test if this is necessary)
-                    real_loss = real_loss.to(self.net.device)
+                    unreduced_losses = []
+                    for k, criterion in enumerate(self.unreduced_criterion_arr):
+                        # redyce = True
+                        target_loss = criterion(outputs[:,ds.out_indexes[k][0]:ds.out_indexes[k][1]], labels[:,ds.out_indexes[k][0]:ds.out_indexes[k][1]])
 
-                    awareness_loss = self.awareness_criterion(awareness, real_loss)
-                    total_loss = self.loss_combination_operator(awareness_loss, loss)
+                        target_loss = target_loss.tolist()
+                        if type(target_loss[0]) == type([]):
+                            target_loss = [np.mean(x) for x in target_loss]
+                        for i, value in enumerate(target_loss):
+                            if len(unreduced_losses) <= i:
+                                unreduced_losses.append([])
+                            unreduced_losses[i].append(value)
+
+                    unreduced_losses = torch.Tensor(unreduced_losses).to(self.net.device)
+
+                    awareness_loss = self.awareness_criterion(awareness,unreduced_losses)
 
                     if CONFIG.MONITORING['batch_loss']:
                         self.monitor.plot_loss(awareness_loss.item(), self.total_iterations, 'Awreness Batch Loss')
-                else:
-                    total_loss = loss
+
 
                 if CONFIG.MONITORING['batch_loss']:
                     self.monitor.plot_loss(loss.item(), self.total_iterations, 'Targets Batch Loss')
 
-                running_loss += total_loss.item()
 
-                total_loss.backward()
+
+                if awareness_loss is not None:
+                    awareness_loss.backward(retain_graph=True)
+
+                loss.backward()
 
                 # @NOTE: Decrease 900 if you want to plot gradients more often, I find it's too expensive to do so
                 if CONFIG.MONITORING['network_heatmap'] and random.randint(0,1000) > 900:
@@ -394,7 +461,7 @@ class NnMixer:
                 error = running_loss / (i + 1)
 
                 if CONFIG.MONITORING['batch_loss']:
-                    self.monitor.plot_loss(total_loss.item(), self.total_iterations, 'Total Batch Loss')
+                    #self.monitor.plot_loss(total_loss.item(), self.total_iterations, 'Total Batch Loss')
                     self.monitor.plot_loss(error, self.total_iterations, 'Mean Total Running Loss')
 
                 if error < 1:
